classdef PerceptionMatLabDemo_exported < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        PerceptionRemoteControlUIFigure  matlab.ui.Figure
        ConnectButton               matlab.ui.control.Button
        Title                       matlab.ui.control.Label
        PerceptionAddressEditFieldLabel  matlab.ui.control.Label
        PerceptionAddressEditField  matlab.ui.control.EditField
        AcquisionControlPanel       matlab.ui.container.Panel
        StartButton                 matlab.ui.control.Button
        StopButton                  matlab.ui.control.Button
        PauseButton                 matlab.ui.control.Button
        TriggerButton               matlab.ui.control.Button
        DisconnectButton            matlab.ui.control.Button
        ConnectionStatus            matlab.ui.control.Label
    end

    
    properties (Access = private)
        Serv = 0; % Perception Server object
        IsConnected = false;
    end
    
    methods (Access = private)
        
        function UpdateUIConnectionStatus(app)
            if (app.IsConnected)
               app.ConnectionStatus.BackgroundColor = [0.4667 0.9686 0.302];
               app.StartButton.Enable = true;
               app.StopButton.Enable = true;
               app.PauseButton.Enable = true;
               app.TriggerButton.Enable = true;
            else
               app.ConnectionStatus.BackgroundColor = [0.090196 0.270588 0.027451];
               app.StartButton.Enable = false;
               app.StopButton.Enable = false;
               app.PauseButton.Enable = false;
               app.TriggerButton.Enable = false;
            end
            
            
        end
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function OnStartup(app)
            UpdateUIConnectionStatus(app);
        end

        % Button pushed function: ConnectButton
        function Connecting(app, event)
            % Create Automation server to the Perception RPC 
            app.Serv = actxserver('PerceptionCom.PerceptionCom');
            % Set the server address to localhost, you can also use the IP address
            app.Serv.SetServerAddress(app.PerceptionAddressEditField.Value);
            % Connect to Perception
            app.Serv.ConnectToServer;
            app.IsConnected = true;
            UpdateUIConnectionStatus(app);
        end

        % Button pushed function: StartButton
        function StartRecording(app, event)
            app.Serv.Start;
        end

        % Button pushed function: StopButton
        function StopRecording(app, event)
             app.Serv.Stop;
        end

        % Button pushed function: PauseButton
        function PausePerception(app, event)
            app.Serv.Pause;
        end

        % Button pushed function: TriggerButton
        function TriggerPerception(app, event)
            app.Serv.Trigger;
        end

        % Button pushed function: DisconnectButton
        function Disconnect(app, event)
            app.Serv.DisconnectFromServer;
            app.IsConnected = false;
            UpdateUIConnectionStatus(app);
        end

        % Close request function: PerceptionRemoteControlUIFigure
        function OnClose(app, event)
            if (app.IsConnected)
                app.Serv.DisconnectFromServer;
            end
            delete(app);
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create PerceptionRemoteControlUIFigure and hide until all components are created
            app.PerceptionRemoteControlUIFigure = uifigure('Visible', 'off');
            app.PerceptionRemoteControlUIFigure.AutoResizeChildren = 'off';
            app.PerceptionRemoteControlUIFigure.Position = [400 400 338 447];
            app.PerceptionRemoteControlUIFigure.Name = 'Perception Remote Control';
            app.PerceptionRemoteControlUIFigure.Resize = 'off';
            app.PerceptionRemoteControlUIFigure.CloseRequestFcn = createCallbackFcn(app, @OnClose, true);
            app.PerceptionRemoteControlUIFigure.Scrollable = 'on';

            % Create ConnectButton
            app.ConnectButton = uibutton(app.PerceptionRemoteControlUIFigure, 'push');
            app.ConnectButton.ButtonPushedFcn = createCallbackFcn(app, @Connecting, true);
            app.ConnectButton.Position = [62 282 213 22];
            app.ConnectButton.Text = 'Connect';

            % Create Title
            app.Title = uilabel(app.PerceptionRemoteControlUIFigure);
            app.Title.HorizontalAlignment = 'center';
            app.Title.FontSize = 18;
            app.Title.FontWeight = 'bold';
            app.Title.Position = [5 357 334 91];
            app.Title.Text = {'Example program showing '; 'how to interact with Perception '; 'from MatLab'};

            % Create PerceptionAddressEditFieldLabel
            app.PerceptionAddressEditFieldLabel = uilabel(app.PerceptionRemoteControlUIFigure);
            app.PerceptionAddressEditFieldLabel.HorizontalAlignment = 'right';
            app.PerceptionAddressEditFieldLabel.Position = [62 319 114 22];
            app.PerceptionAddressEditFieldLabel.Text = 'Perception Address:';

            % Create PerceptionAddressEditField
            app.PerceptionAddressEditField = uieditfield(app.PerceptionRemoteControlUIFigure, 'text');
            app.PerceptionAddressEditField.Position = [191 319 84 22];
            app.PerceptionAddressEditField.Value = 'LocalHost';

            % Create AcquisionControlPanel
            app.AcquisionControlPanel = uipanel(app.PerceptionRemoteControlUIFigure);
            app.AcquisionControlPanel.AutoResizeChildren = 'off';
            app.AcquisionControlPanel.TitlePosition = 'centertop';
            app.AcquisionControlPanel.Title = 'Acquision Control';
            app.AcquisionControlPanel.Position = [104 28 130 173];

            % Create StartButton
            app.StartButton = uibutton(app.AcquisionControlPanel, 'push');
            app.StartButton.ButtonPushedFcn = createCallbackFcn(app, @StartRecording, true);
            app.StartButton.Enable = 'off';
            app.StartButton.Position = [15 115 100 22];
            app.StartButton.Text = 'Start';

            % Create StopButton
            app.StopButton = uibutton(app.AcquisionControlPanel, 'push');
            app.StopButton.ButtonPushedFcn = createCallbackFcn(app, @StopRecording, true);
            app.StopButton.Enable = 'off';
            app.StopButton.Position = [15 82 100 22];
            app.StopButton.Text = 'Stop';

            % Create PauseButton
            app.PauseButton = uibutton(app.AcquisionControlPanel, 'push');
            app.PauseButton.ButtonPushedFcn = createCallbackFcn(app, @PausePerception, true);
            app.PauseButton.Enable = 'off';
            app.PauseButton.Position = [15 49 100 22];
            app.PauseButton.Text = 'Pause';

            % Create TriggerButton
            app.TriggerButton = uibutton(app.AcquisionControlPanel, 'push');
            app.TriggerButton.ButtonPushedFcn = createCallbackFcn(app, @TriggerPerception, true);
            app.TriggerButton.Enable = 'off';
            app.TriggerButton.Position = [15 17 100 22];
            app.TriggerButton.Text = 'Trigger';

            % Create DisconnectButton
            app.DisconnectButton = uibutton(app.PerceptionRemoteControlUIFigure, 'push');
            app.DisconnectButton.ButtonPushedFcn = createCallbackFcn(app, @Disconnect, true);
            app.DisconnectButton.Position = [62 237 213 22];
            app.DisconnectButton.Text = 'Disconnect';

            % Create ConnectionStatus
            app.ConnectionStatus = uilabel(app.PerceptionRemoteControlUIFigure);
            app.ConnectionStatus.BackgroundColor = [0.0902 0.2706 0.0275];
            app.ConnectionStatus.Position = [124 260 100 20];
            app.ConnectionStatus.Text = '';

            % Show the figure after all components are created
            app.PerceptionRemoteControlUIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = PerceptionMatLabDemo_exported

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.PerceptionRemoteControlUIFigure)

            % Execute the startup function
            runStartupFcn(app, @OnStartup)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.PerceptionRemoteControlUIFigure)
        end
    end
end